@@ -3,8 +3,11 @@
 from PyQt4 import QtCore, QtGui
 from qtGUI.qtUImain import Ui_MainWindow
 from qtGUI.aboutDialog import Ui_aboutDialog
+from qtGUI.instrumentDialog import Ui_instrumentDialog
+from mingus.containers.Instrument import MidiInstrument
 from sys import argv
 from os import path
+import Musicians
 
 import Options
 from mingus.core import diatonic
@@ -23,6 +26,75 @@ class OptionClass:
 	meter = (4,4)
 	pass
 
+class InstrumentDialog(QtGui.QDialog):
+
+	def __init__(self):
+		QtGui.QDialog.__init__(self)
+		self.ui = Ui_instrumentDialog()
+		self.ui.setupUi(self)
+		self.setup()
+
+	def setup(self):
+		for x in Options.get_available_instruments():
+			self.ui.algorithm.addItem(x)
+
+		m = MidiInstrument()
+		d = 1
+		for x in m.names:
+			self.ui.midi.addItem("%d. %s" % (d, x))
+			d += 1
+
+	def load_instrument(self, instr_str):
+		parts = instr_str.split(" ")
+
+		index = self.combo_index_by_text(self.ui.algorithm, parts[0])
+		if index > 0:
+			self.ui.algorithm.setCurrentIndex(index)
+
+		i = getattr(Musicians, parts[0])({})
+		self.ui.stepstart.setValue(i.start)
+		self.ui.stepend.setValue(i.end)
+		self.ui.step.setValue(i.step)
+		if 'max_velocity' in i.params:
+			self.ui.maxvelocity.setValue(i.params["max_velocity"])
+		else:
+			self.ui.maxvelocity.setValue(100)
+		if 'min_velocity' in i.params:
+			self.ui.minvelocity.setValue(i.params["min_velocity"])
+		else:
+			self.ui.minvelocity.setValue(50)
+
+		self.ui.channel.setEnabled(True)
+		if 'channel' in i.params:
+			self.ui.channel.setValue(i.params["channel"])
+			if i.params["channel"] == 9:
+				self.ui.channel.setEnabled(False)
+		else:
+			self.ui.channel.setValue(0)
+
+		if 'let_ring' in i.params and i.params["let_ring"]:
+			self.ui.letring.setChecked(True)
+			self.ui.noteduration.setEnabled(True)
+			if 'note_length' in i.params:
+				self.ui.noteduration = i.params["note_length"]
+			else:
+				self.ui.noteduration = 2
+		else:
+			self.ui.letring.setChecked(False)
+			self.ui.noteduration.setEnabled(False)
+
+		if 'midi_instr' in i.params:
+			self.ui.midi.setCurrentIndex(i.params['midi_instr'])
+		else:
+			self.ui.midi.setCurrentIndex(0)
+
+
+	def combo_index_by_text(self, combo, text):
+		for x in range(combo.count()):
+			if combo.itemText(x) == text:
+				return x
+		return -1
+
 class ImproviserMainWindow(QtGui.QMainWindow):
 
 	lastfile = ""
@@ -88,6 +160,9 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.connect(self.ui.removeinstrument,
 			QtCore.SIGNAL("clicked()"),
 			self.remove_instrument)
+		self.connect(self.ui.editinstrument,
+			QtCore.SIGNAL("clicked()"),
+			self.edit_instrument)
 		self.connect(self.ui.removeprogression,
 			QtCore.SIGNAL("clicked()"),
 			self.remove_progression)
@@ -145,6 +220,15 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		if i >= 0:
 			self.ui.instruments.takeItem(i)
 
+	def edit_instrument(self):
+		cur = self.ui.instruments.item(self.ui.instruments.currentRow())
+		if cur is None:
+			return
+		i = InstrumentDialog()
+		i.load_instrument(str(cur.text()))
+		i.show()
+		i.exec_()
+
 	def get_file_name(self, name, n = 0):
 		if n == 0:
 			midi = name + ".mid"
